@@ -84,6 +84,10 @@ class RecordBatch:
         tab._recordbatch = pyt
         return tab
 
+    @staticmethod
+    def _from_series(series: list[Series]) -> RecordBatch:
+        return RecordBatch._from_pyrecordbatch(_PyRecordBatch.from_pyseries_list([s._series for s in series]))
+
     @staticmethod
     def from_arrow_table(arrow_table: pa.Table) -> RecordBatch:
         assert isinstance(arrow_table, pa.Table)
@@ -462,8 +466,8 @@ class RecordBatch:
 
     def __reduce__(
         self,
-    ) -> tuple[Callable[[dict[str, Any]], RecordBatch], tuple[dict[str, Series]]]:
-        return RecordBatch.from_pydict, ({column.name(): column for column in self.columns()},)
+    ) -> tuple[Callable[[list[Series]], RecordBatch], tuple[list[Series]]]:
+        return RecordBatch._from_series, (self.columns(),)
 
     @classmethod
     def read_parquet(
@@ -528,7 +532,7 @@ class RecordBatch:
         multithreaded_io: bool | None = None,
     ) -> RecordBatch:
         if not isinstance(paths, Series):
-            paths = Series.from_pylist(paths, name="uris").cast(DataType.string())
+            paths = Series.from_pylist(paths, name="uris", dtype=DataType.string())
         assert paths.name() == "uris", f"Expected input series to have name 'uris', but found: {paths.name()}"
         return RecordBatch._from_pyrecordbatch(
             _read_parquet_statistics(
