@@ -176,17 +176,14 @@ def run_udf(
         return result_series.rename(name).cast(return_dtype)._series
     elif isinstance(results[0], list):
         result_list = [x for res in results for x in res]
-        if return_dtype == DataType.python():
-            return Series.from_pylist(result_list, name=name, pyobj="force")._series
-        else:
-            return Series.from_pylist(result_list, name=name, pyobj="allow").cast(return_dtype)._series
+        return Series.from_pylist(result_list, name=name, dtype=return_dtype)._series
     elif np.module_available() and isinstance(results[0], np.ndarray):  # type: ignore[attr-defined]
         np_results = cast("list[np.ndarray[Any, Any]]", results)
         result_np = np.concatenate(np_results)
-        return Series.from_numpy(result_np, name=name).cast(return_dtype)._series
+        return Series.from_numpy(result_np, name=name, dtype=return_dtype)._series
     elif pa.module_available() and isinstance(results[0], (pa.Array, pa.ChunkedArray)):
         result_pa = pa.concat_arrays(results)
-        return Series.from_arrow(result_pa, name=name).cast(return_dtype)._series
+        return Series.from_arrow(result_pa, name=name, dtype=return_dtype)._series
     else:
         raise NotImplementedError(
             f"Return type {type(results[0])} not supported for UDF {func}, expected daft.Series, list, np.ndarray, or pa.Array containing {return_dtype}"
@@ -567,3 +564,109 @@ def udf(
         return udf
 
     return _udf
+
+
+class _DaftFunc:
+    """`@daft.func` Decorator to convert a Python function into a `UDF`.
+
+    Unlike `@daft.udf(...)`, `@daft.func` operates on single values instead of batches.
+
+    Examples:
+    >>> import daft
+    >>> from daft import col
+    >>> @daft.func  # or @daft.func()
+    ... # or you can specify the return type like our existing @daft.udf
+    ... # @daft.func(return_dtype=daft.DataType.int64())
+    ... def my_sum(a: int, b: int) -> int:
+    ...     return a + b
+    >>>
+    >>> df = daft.from_pydict({"x": [1, 2, 3], "y": [4, 5, 6]})
+    >>> df.select(my_sum(col("x"), col("y"))).collect()
+    ╭───────╮
+    │ x     │
+    │ ---   │
+    │ Int64 │
+    ╞═══════╡
+    │ 5     │
+    ├╌╌╌╌╌╌╌┤
+    │ 7     │
+    ├╌╌╌╌╌╌╌┤
+    │ 9     │
+    ╰───────╯
+    <BLANKLINE>
+    (Showing first 3 of 3 rows)
+    """
+
+    batch = udf
+
+    def __init__(self, func: UserDefinedPyFuncLike | None = None, *args: Any, **kwargs: Any) -> None:
+        self._func = func
+        self._args = args
+        self._kwargs = kwargs
+
+    def get_return_type_for_func(self, f: UserDefinedPyFuncLike) -> DataType:
+        try:
+            from typing import get_type_hints
+        except ImportError:
+            if self._kwargs.get("return_dtype") is None:
+                raise ImportError("return_dtype is required when function has no return annotation")
+
+        if self._kwargs.get("return_dtype") is None:
+            type_hints = get_type_hints(f)
+            return_annotation = type_hints.get("return")
+            if return_annotation is None:
+                raise ValueError("return_dtype is required when function has no return annotation")
+            inferred_return_dtype = DataType._infer_type(return_annotation)
+        else:
+            inferred_return_dtype = DataType._infer_type(self._kwargs.get("return_dtype"))  # type: ignore
+        return inferred_return_dtype
+
+    def __call__(self, *args: Any, **kwargs: Any) -> Expression | UDF:
+        self._args = self._args if self._args else args
+        self._kwargs = self._kwargs if self._kwargs else kwargs
+        args = self._args
+        kwargs = self._kwargs
+
+        # if None, decorator was called with `@daft.func()` syntax
+        # otherwise it was called with `@daft.func` syntax
+        if self._func is None:
+
+            def _udf(f: UserDefinedPyFuncLike) -> UDF:
+                inferred_return_dtype = self.get_return_type_for_func(f)
+                # Grab a name for the UDF. It **should** be unique.
+                module_name = getattr(f, "__module__", "")
+                qual_name = getattr(f, "__qualname__")
+
+                if module_name:
+                    name = f"{module_name}.{qual_name}"
+                else:
+                    name = qual_name
+
+                def batch_func(*series: Series) -> list[Any]:
+                    return [f(*args) for args in zip(*series)]
+
+                udf = UDF(
+                    inner=batch_func,
+                    name=name,
+                    return_dtype=inferred_return_dtype,
+                )
+
+                return udf
+
+            return _udf(args[0])
+        else:
+
+            def batch_func(*series: Series) -> list[Any]:
+                return [self._func(*args) for args in zip(*series)]  # type: ignore
+
+            inferred_return_dtype = self.get_return_type_for_func(self._func)
+            name = getattr(self._func, "__module__", "")
+            if name:
+                name = name + "."
+            name = name + getattr(self._func, "__qualname__")
+
+            return UDF(
+                inner=batch_func,
+                name=name,
+                return_dtype=inferred_return_dtype,
+            )(*args)
