@@ -164,8 +164,9 @@ impl ActorUDF {
 
     #[allow(clippy::too_many_arguments)]
     pub fn new(
-        stage_config: &StageConfig,
         node_id: NodeID,
+        logical_node_id: Option<NodeID>,
+        stage_config: &StageConfig,
         projection: Vec<BoundExpr>,
         batch_size: Option<usize>,
         memory_request: u64,
@@ -178,8 +179,13 @@ impl ActorUDF {
             Self::NODE_NAME,
             vec![child.node_id()],
             vec![child.name()],
+            logical_node_id,
+        );
+        let config = PipelineNodeConfig::new(
+            schema,
+            stage_config.config.clone(),
+            child.config().clustering_spec.clone(),
         );
-        let config = PipelineNodeConfig::new(schema, stage_config.config.clone());
         Ok(Self {
             config,
             context,
@@ -219,12 +225,12 @@ impl ActorUDF {
                     };
 
                     let task = self.make_actor_udf_task_for_materialized_outputs(
-                        vec![materialized_output],
+                        materialized_output,
                         worker_id,
                         actors,
                         TaskContext::from((&self.context, task_id_counter.next())),
                     )?;
-                    let (submittable_task, notify_token) = task.with_notify_token();
+                    let (submittable_task, notify_token) = task.add_notify_token();
                     running_tasks.spawn(notify_token);
                     if result_tx
                         .send(PipelineOutput::Task(submittable_task))
@@ -250,7 +256,7 @@ impl ActorUDF {
                         actors,
                         TaskContext::from((&self.context, task_id_counter.next())),
                     )?;
-                    let (submittable_task, notify_token) = modified_task.with_notify_token();
+                    let (submittable_task, notify_token) = modified_task.add_notify_token();
                     running_tasks.spawn(notify_token);
                     if result_tx
                         .send(PipelineOutput::Task(submittable_task))
@@ -275,17 +281,13 @@ impl ActorUDF {
 
     fn make_actor_udf_task_for_materialized_outputs(
         &self,
-        materialized_outputs: Vec<MaterializedOutput>,
+        materialized_output: MaterializedOutput,
         worker_id: WorkerId,
         actors: Vec<PyObjectWrapper>,
         task_context: TaskContext,
     ) -> DaftResult<SubmittableTask<SwordfishTask>> {
         // Extract all partitions from materialized outputs
-        let mut partitions = Vec::new();
-        for materialized_output in materialized_outputs {
-            let (partition, _) = materialized_output.into_inner();
-            partitions.push(partition);
-        }
+        let partitions = materialized_output.partitions().to_vec();
 
         let in_memory_info = InMemoryInfo::new(
             self.config.schema.clone(),
