@@ -10,7 +10,7 @@ use common_display::{
 use common_error::DaftResult;
 use common_partitioning::PartitionRef;
 use daft_local_plan::{LocalPhysicalPlan, LocalPhysicalPlanRef};
-use daft_logical_plan::{stats::StatsState, InMemoryInfo};
+use daft_logical_plan::{partitioning::ClusteringSpecRef, stats::StatsState, InMemoryInfo};
 use daft_schema::schema::SchemaRef;
 use futures::{Stream, StreamExt};
 use itertools::Itertools;
@@ -29,17 +29,27 @@ use crate::{
 
 #[cfg(feature = "python")]
 mod actor_udf;
+mod aggregate;
+mod concat;
+mod distinct;
 mod explode;
 mod filter;
+mod gather;
+mod hash_join;
 mod in_memory_source;
 mod limit;
 pub(crate) mod materialize;
+mod monotonically_increasing_id;
 mod project;
+mod repartition;
 mod sample;
 mod scan_source;
 mod sink;
+mod sort;
+mod top_n;
 mod translate;
 mod unpivot;
+mod window;
 
 pub(crate) use translate::logical_plan_to_pipeline_node;
 pub(crate) type NodeID = u32;
@@ -51,19 +61,19 @@ pub(crate) type NodeName = &'static str;
 /// to schedule follow-up pipeline nodes on the same worker.
 #[derive(Clone, Debug)]
 pub(crate) struct MaterializedOutput {
-    partition: PartitionRef,
+    partition: Vec<PartitionRef>,
     worker_id: WorkerId,
 }
 
 impl MaterializedOutput {
-    pub fn new(partition: PartitionRef, worker_id: WorkerId) -> Self {
+    pub fn new(partition: Vec<PartitionRef>, worker_id: WorkerId) -> Self {
         Self {
             partition,
             worker_id,
         }
     }
 
-    pub fn partition(&self) -> &PartitionRef {
+    pub fn partitions(&self) -> &[PartitionRef] {
         &self.partition
     }
 
@@ -72,9 +82,30 @@ impl MaterializedOutput {
         &self.worker_id
     }
 
-    pub fn into_inner(self) -> (PartitionRef, WorkerId) {
+    pub fn into_inner(self) -> (Vec<PartitionRef>, WorkerId) {
         (self.partition, self.worker_id)
     }
+
+    pub fn split_into_materialized_outputs(&self) -> Vec<Self> {
+        self.partition
+            .iter()
+            .map(|partition| Self::new(vec![partition.clone()], self.worker_id.clone()))
+            .collect()
+    }
+
+    pub fn num_rows(&self) -> DaftResult<usize> {
+        self.partition
+            .iter()
+            .map(|partition| partition.num_rows())
+            .sum()
+    }
+
+    pub fn size_bytes(&self) -> DaftResult<usize> {
+        self.partition
+            .iter()
+            .map(|partition| partition.size_bytes().map(|size| size.unwrap_or(0)))
+            .sum()
+    }
 }
 
 #[derive(Debug)]
@@ -88,15 +119,19 @@ pub(crate) enum PipelineOutput<T: Task> {
 pub(super) struct PipelineNodeConfig {
     pub schema: SchemaRef,
     pub execution_config: Arc<DaftExecutionConfig>,
-    // TODO: add clustering spec, may be useful for determining shuffles
-    // pub clustering_spec: ClusteringSpecRef,
+    pub clustering_spec: ClusteringSpecRef,
 }
 
 impl PipelineNodeConfig {
-    pub fn new(schema: SchemaRef, execution_config: Arc<DaftExecutionConfig>) -> Self {
+    pub fn new(
+        schema: SchemaRef,
+        execution_config: Arc<DaftExecutionConfig>,
+        clustering_spec: ClusteringSpecRef,
+    ) -> Self {
         Self {
             schema,
             execution_config,
+            clustering_spec,
         }
     }
 }
@@ -108,6 +143,7 @@ pub(super) struct PipelineNodeContext {
     pub node_name: NodeName,
     pub child_ids: Vec<NodeID>,
     pub child_names: Vec<NodeName>,
+    pub logical_node_id: Option<NodeID>,
 }
 
 impl PipelineNodeContext {
@@ -117,6 +153,7 @@ impl PipelineNodeContext {
         node_name: NodeName,
         child_ids: Vec<NodeID>,
         child_names: Vec<NodeName>,
+        logical_node_id: Option<NodeID>,
     ) -> Self {
         Self {
             plan_id: stage_config.plan_id,
@@ -125,6 +162,7 @@ impl PipelineNodeContext {
             node_name,
             child_ids,
             child_names,
+            logical_node_id,
         }
     }
 
@@ -136,6 +174,10 @@ impl PipelineNodeContext {
             ("node_name".to_string(), self.node_name.to_string()),
             ("child_ids".to_string(), self.child_ids.iter().join(",")),
             ("child_names".to_string(), self.child_names.iter().join(",")),
+            (
+                "logical_node_id".to_string(),
+                self.logical_node_id.unwrap_or(0).to_string(),
+            ),
         ])
     }
 }
@@ -296,10 +338,10 @@ where
     let mut worker_id_counts: HashMap<WorkerId, usize> = HashMap::new();
 
     for materialized_output in materialized_outputs {
-        let (partition_ref, worker_id) = materialized_output.into_inner();
-        total_size_bytes += partition_ref.size_bytes()?.unwrap_or(0);
-        total_num_rows += partition_ref.num_rows().unwrap_or(0);
-        partition_refs.push(partition_ref);
+        total_size_bytes += materialized_output.size_bytes()?;
+        total_num_rows += materialized_output.num_rows()?;
+        let (output_refs, worker_id) = materialized_output.into_inner();
+        partition_refs.extend(output_refs);
         let count = worker_id_counts.entry(worker_id.clone()).or_insert(0);
         *count += 1;
     }
@@ -332,13 +374,23 @@ where
         psets,
         SchedulingStrategy::WorkerAffinity {
             worker_id,
-            soft: false,
+            soft: true,
         },
         node.context().to_hashmap(),
     );
     Ok(SubmittableTask::new(task))
 }
 
+fn make_in_memory_scan_from_materialized_outputs(
+    task_context: TaskContext,
+    materialized_outputs: Vec<MaterializedOutput>,
+    node: &Arc<dyn DistributedPipelineNode>,
+) -> DaftResult<SubmittableTask<SwordfishTask>> {
+    make_new_task_from_materialized_outputs(task_context, materialized_outputs, node, &|input| {
+        Ok(input)
+    })
+}
+
 fn append_plan_to_existing_task<F>(
     task_context: TaskContext,
     submittable_task: SubmittableTask<SwordfishTask>,
