@@ -1,13 +1,13 @@
 use std::collections::{BinaryHeap, HashMap};
 
-use super::{SchedulableTask, ScheduledTask, Scheduler, WorkerSnapshot};
+use super::{PendingTask, ScheduledTask, Scheduler, WorkerSnapshot};
 use crate::scheduling::{
     task::{SchedulingStrategy, Task, TaskDetails},
     worker::WorkerId,
 };
 
 pub(super) struct DefaultScheduler<T: Task> {
-    pending_tasks: BinaryHeap<SchedulableTask<T>>,
+    pending_tasks: BinaryHeap<PendingTask<T>>,
     worker_snapshots: HashMap<WorkerId, WorkerSnapshot>,
 }
 
@@ -58,7 +58,7 @@ impl<T: Task> DefaultScheduler<T> {
         }
     }
 
-    fn try_schedule_task(&self, task: &SchedulableTask<T>) -> Option<WorkerId> {
+    fn try_schedule_task(&self, task: &PendingTask<T>) -> Option<WorkerId> {
         match task.strategy() {
             SchedulingStrategy::Spread => self.try_schedule_spread_task(&task.task),
             SchedulingStrategy::WorkerAffinity { worker_id, soft } => {
@@ -69,7 +69,7 @@ impl<T: Task> DefaultScheduler<T> {
 }
 
 impl<T: Task> Scheduler<T> for DefaultScheduler<T> {
-    fn enqueue_tasks(&mut self, tasks: Vec<SchedulableTask<T>>) {
+    fn enqueue_tasks(&mut self, tasks: Vec<PendingTask<T>>) {
         self.pending_tasks.extend(tasks);
     }
 
@@ -77,7 +77,7 @@ impl<T: Task> Scheduler<T> for DefaultScheduler<T> {
     // However, this can cause the scheduler to have too many pending tasks, creating a bottleneck in scheduling.
     // Potentially, we should allow workers to maintain a backlog queue of tasks, and automatically run them when they have capacity.
     // Key thing is that this should be profiled and tested.
-    fn get_schedulable_tasks(&mut self) -> Vec<ScheduledTask<T>> {
+    fn schedule_tasks(&mut self) -> Vec<ScheduledTask<T>> {
         let mut scheduled = Vec::new();
         let mut unscheduled = Vec::new();
         while let Some(task) = self.pending_tasks.pop() {
@@ -161,7 +161,7 @@ mod tests {
 
         // Enqueue and schedule tasks
         scheduler.enqueue_tasks(initial_tasks);
-        let result = scheduler.get_schedulable_tasks();
+        let result = scheduler.schedule_tasks();
 
         // All tasks should be scheduled because there is enough capacity
         assert_eq!(result.len(), 3);
@@ -204,7 +204,7 @@ mod tests {
 
         // Enqueue and schedule tasks
         scheduler.enqueue_tasks(initial_tasks);
-        let result = scheduler.get_schedulable_tasks();
+        let result = scheduler.schedule_tasks();
 
         // All tasks should be scheduled because there is enough capacity
         assert_eq!(result.len(), 3);
@@ -245,7 +245,7 @@ mod tests {
         ];
 
         scheduler.enqueue_tasks(tasks);
-        let result = scheduler.get_schedulable_tasks();
+        let result = scheduler.schedule_tasks();
 
         // 2 tasks should be scheduled
         assert_eq!(result.len(), 2);
@@ -270,7 +270,7 @@ mod tests {
         ];
 
         scheduler.enqueue_tasks(tasks);
-        let result = scheduler.get_schedulable_tasks();
+        let result = scheduler.schedule_tasks();
 
         // Only 2 tasks should be scheduled, because worker 3 has 2 slots available
         assert_eq!(result.len(), 2);
@@ -302,7 +302,7 @@ mod tests {
         ];
 
         scheduler.enqueue_tasks(tasks);
-        let scheduled_tasks = scheduler.get_schedulable_tasks();
+        let scheduled_tasks = scheduler.schedule_tasks();
 
         // 3 tasks should be scheduled, 1 for each worker
         assert_eq!(scheduled_tasks.len(), 3);
@@ -325,7 +325,7 @@ mod tests {
         ];
 
         scheduler.enqueue_tasks(tasks);
-        let scheduled_tasks = scheduler.get_schedulable_tasks();
+        let scheduled_tasks = scheduler.schedule_tasks();
 
         // worker 1 should not be available, worker 2 should have 1 slot available, worker 3 should have 2 slots available
         assert_eq!(scheduled_tasks.len(), 2);
@@ -359,7 +359,7 @@ mod tests {
             .collect();
 
         scheduler.enqueue_tasks(tasks);
-        let result = scheduler.get_schedulable_tasks();
+        let result = scheduler.schedule_tasks();
 
         // Only 2 tasks should be scheduled (one per worker)
         assert_eq!(result.len(), 2);
@@ -371,7 +371,7 @@ mod tests {
         scheduler.enqueue_tasks(vec![high_priority_task]);
 
         // The high-priority task should not be scheduled because worker1 is full
-        let result = scheduler.get_schedulable_tasks();
+        let result = scheduler.schedule_tasks();
         assert_eq!(result.len(), 0);
         assert_eq!(scheduler.num_pending_tasks(), 99);
 
@@ -382,7 +382,7 @@ mod tests {
         scheduler.update_worker_state(&[new_worker_snapshot]);
 
         // The high-priority task should now be scheduled to the new worker
-        let result = scheduler.get_schedulable_tasks();
+        let result = scheduler.schedule_tasks();
         assert_eq!(result.len(), 1);
         assert_eq!(scheduler.num_pending_tasks(), 98);
         assert_eq!(result[0].worker_id, worker_3);
@@ -430,7 +430,7 @@ mod tests {
         ];
 
         scheduler.enqueue_tasks(tasks);
-        let result = scheduler.get_schedulable_tasks();
+        let result = scheduler.schedule_tasks();
 
         assert_eq!(result.len(), 3);
         assert_eq!(scheduler.num_pending_tasks(), 0);
@@ -499,7 +499,7 @@ mod tests {
         ];
 
         scheduler.enqueue_tasks(tasks);
-        let result = scheduler.get_schedulable_tasks();
+        let result = scheduler.schedule_tasks();
 
         assert_eq!(result.len(), 3);
         assert_eq!(scheduler.num_pending_tasks(), 0);
@@ -524,7 +524,7 @@ mod tests {
         ];
 
         scheduler.enqueue_tasks(tasks);
-        let result = scheduler.get_schedulable_tasks();
+        let result = scheduler.schedule_tasks();
 
         assert_eq!(result.len(), 0);
         assert_eq!(scheduler.num_pending_tasks(), 2);
@@ -552,7 +552,7 @@ mod tests {
         ];
 
         scheduler.enqueue_tasks(tasks);
-        let result = scheduler.get_schedulable_tasks();
+        let result = scheduler.schedule_tasks();
 
         // Only 2 tasks should be scheduled (1 per worker)
         assert_eq!(result.len(), 2);
@@ -577,7 +577,7 @@ mod tests {
         let tasks = vec![create_spread_task(Some(1))];
 
         scheduler.enqueue_tasks(tasks);
-        let result = scheduler.get_schedulable_tasks();
+        let result = scheduler.schedule_tasks();
 
         assert_eq!(result.len(), 0);
         assert_eq!(scheduler.num_pending_tasks(), 1);
