@@ -20,7 +20,7 @@ impl ScalarUDF for LStrip {
         "lstrip"
     }
 
-    fn evaluate(&self, inputs: daft_dsl::functions::FunctionArgs<Series>) -> DaftResult<Series> {
+    fn call(&self, inputs: daft_dsl::functions::FunctionArgs<Series>) -> DaftResult<Series> {
         unary_utf8_evaluate(inputs, |s| {
             s.with_utf8_array(|arr| {
                 arr.unary_broadcasted_op(|val| val.trim_start().into())
@@ -29,7 +29,7 @@ impl ScalarUDF for LStrip {
         })
     }
 
-    fn function_args_to_field(
+    fn get_return_field(
         &self,
         inputs: FunctionArgs<ExprRef>,
         schema: &Schema,
