@@ -14,17 +14,13 @@ impl ScalarUDF for MonotonicallyIncreasingId {
     fn name(&self) -> &'static str {
         "monotonically_increasing_id"
     }
-    fn evaluate(&self, _: daft_dsl::functions::FunctionArgs<Series>) -> DaftResult<Series> {
+    fn call(&self, _: daft_dsl::functions::FunctionArgs<Series>) -> DaftResult<Series> {
         Err(DaftError::NotImplemented(
             "monotonically_increasing_id should be rewritten into a separate plan step by the optimizer. If you're seeing this error, the DetectMonotonicId optimization rule may not have been applied.".to_string(),
         ))
     }
 
-    fn function_args_to_field(
-        &self,
-        inputs: FunctionArgs<ExprRef>,
-        _: &Schema,
-    ) -> DaftResult<Field> {
+    fn get_return_field(&self, inputs: FunctionArgs<ExprRef>, _: &Schema) -> DaftResult<Field> {
         if !inputs.is_empty() {
             return Err(DaftError::ValueError(format!(
                 "Expected 0 input args, got {}",
