@@ -143,12 +143,26 @@ impl ObjectSource for LocalSource {
         &self,
         uri: &str,
         range: Option<Range<usize>>,
-        _io_stats: Option<IOStatsRef>,
+        io_stats: Option<IOStatsRef>,
     ) -> super::Result<GetResult> {
         const LOCAL_PROTOCOL: &str = "file://";
-        if let Some(uri) = uri.strip_prefix(LOCAL_PROTOCOL) {
+        if let Some(file) = uri.strip_prefix(LOCAL_PROTOCOL) {
+            if let Some(range) = range.clone() {
+                let size = self.get_size(uri, io_stats).await?;
+                if range.start >= size {
+                    return Err(Error::UnableToReadBytes {
+                        path: file.into(),
+                        source: std::io::Error::new(
+                            std::io::ErrorKind::InvalidInput,
+                            "Range start is larger than file size",
+                        ),
+                    }
+                    .into());
+                }
+            }
+
             Ok(GetResult::File(LocalFile {
-                path: uri.into(),
+                path: file.into(),
                 range,
             }))
         } else {
@@ -385,6 +399,7 @@ mod tests {
     use std::{default, io::Write};
 
     use crate::{
+        integrations::test_full_get,
         object_io::{FileMetadata, FileType, ObjectSource},
         HttpSource, LocalSource, Result,
     };
@@ -414,46 +429,7 @@ mod tests {
         let parquet_file_path = format!("file://{}", file1.path().to_str().unwrap());
         let client = LocalSource::get_client().await?;
 
-        let try_all_bytes = client
-            .get(&parquet_file_path, None, None)
-            .await?
-            .bytes()
-            .await?;
-        assert_eq!(try_all_bytes.len(), bytes.len());
-        assert_eq!(try_all_bytes, bytes);
-
-        let first_bytes = client
-            .get(&parquet_file_path, Some(0..10), None)
-            .await?
-            .bytes()
-            .await?;
-        assert_eq!(first_bytes.len(), 10);
-        assert_eq!(first_bytes.as_ref(), &bytes[..10]);
-
-        let first_bytes = client
-            .get(&parquet_file_path, Some(10..100), None)
-            .await?
-            .bytes()
-            .await?;
-        assert_eq!(first_bytes.len(), 90);
-        assert_eq!(first_bytes.as_ref(), &bytes[10..100]);
-
-        let last_bytes = client
-            .get(
-                &parquet_file_path,
-                Some((bytes.len() - 10)..(bytes.len() + 10)),
-                None,
-            )
-            .await?
-            .bytes()
-            .await?;
-        assert_eq!(last_bytes.len(), 10);
-        assert_eq!(last_bytes.as_ref(), &bytes[(bytes.len() - 10)..]);
-
-        let size_from_get_size = client.get_size(parquet_file_path.as_str(), None).await?;
-        assert_eq!(size_from_get_size, bytes.len());
-
-        Ok(())
+        test_full_get(client, &parquet_file_path, &bytes).await
     }
 
     #[tokio::test]
