@@ -20,6 +20,7 @@ use crate::{
 #[derive(Clone, Debug, PartialEq, Eq, Hash, Serialize, Deserialize)]
 pub struct Project {
     pub plan_id: Option<usize>,
+    pub node_id: Option<usize>,
     // Upstream node.
     pub input: Arc<LogicalPlan>,
     pub projection: Vec<ExprRef>,
@@ -49,6 +50,7 @@ impl Project {
 
         Ok(Self {
             plan_id: None,
+            node_id: None,
             input: factored_input,
             projection: factored_projection,
             projected_schema,
@@ -61,6 +63,11 @@ impl Project {
         self
     }
 
+    pub fn with_node_id(mut self, node_id: usize) -> Self {
+        self.node_id = Some(node_id);
+        self
+    }
+
     /// Create a new Projection using the specified output schema
     pub(crate) fn new_from_schema(
         input: Arc<LogicalPlan>,
