@@ -215,6 +215,12 @@ impl PyMicroPartition {
         py.allow_threads(|| Ok(MicroPartition::concat(mps_iter)?.into()))
     }
 
+    #[staticmethod]
+    pub fn concat_or_empty(py: Python, to_concat: Vec<Self>, schema: PySchema) -> PyResult<Self> {
+        let mps_iter = to_concat.iter().map(|t| t.inner.as_ref());
+        py.allow_threads(|| Ok(MicroPartition::concat_or_empty(mps_iter, schema.schema)?.into()))
+    }
+
     pub fn slice(&self, py: Python, start: i64, end: i64) -> PyResult<Self> {
         py.allow_threads(|| Ok(self.inner.slice(start as usize, end as usize)?.into()))
     }
