@@ -9,10 +9,13 @@ import pandas as pd
 import pytest
 
 from daft import DataType, Window, col
+from daft.context import get_context
 from tests.conftest import assert_df_equals, get_tests_daft_runner_name
 
 pytestmark = pytest.mark.skipif(
-    get_tests_daft_runner_name() != "native", reason="Window tests only run on native runner"
+    get_tests_daft_runner_name() == "ray"
+    and get_context().daft_execution_config.use_experimental_distributed_engine is False,
+    reason="requires Native Runner or Flotilla to be in use",
 )
 
 
